==> MeatAxe-2.3.0.tar.gz: src/isisom.c <==
/* ============================= C MeatAxe ==================================
   File:        $Id: isisom.c,v 1.2 1998/06/25 19:02:33 mringe Exp $
   Comment:     Isomorphism test.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"
#include <string.h>


/*{{{ --------------------------------------------------------------------------
!function IsIsomorphic "Compare representations"
!section reps
!synopsis 
    int IsIsomorphic(int ngen, const matrix_t *gen1[], const CfInfo *info1, 
	const matrix_t *gen2[], matrix_t **trans)
!arg ngen
    Number of generators.
!arg gen1
    Generators for the first representation.
!arg info1
    Pointer to an |CfInfo| structure for the first representation.
!arg gen2
    Generators for the second representation.
!arg trans
    Buffer for basis transformation matrix, or |NULL|.
!return
    1 if the representations are isomorphic, 0 if they are not isomorphic, and
    $-1$ on error.
!description
    This function decides if two irreducible representations are isomorphic.
    To compare the representations, the function needs an identifying word
    for the first representation, i.e., the fields |info1->idword|, 
    |info1->idpol| and |info1->spl| must be set, and the generators in 
    |gen1| must be in standard basis with respect to the identifying word.

    If the representations are isomoprhic, and |trans| is not |NULL|, the
    basis transformation which makes the second representation identical
    to the first is stored into |*trans|. To be more precise, if $g_i$ is 
    the matrix in |gen1[i]|, $h_i$ the matrix in |gen2[i]|, and $T$ the 
    matrix returned in |trans|, then $Th_i T^{-1}=g_i$.
!seealso 
-------------------------------------------------------------------------- }}}*/

int IsIsomorphic(int ngen, const matrix_t *gen1[], const CfInfo *info1,
    const matrix_t *gen2[], matrix_t **trans)

{
    MTXCONTEXT(IsIsomorphic);
    int j;
    wgdata_t *wg;
    matrix_t *word, *m, *seed, *b, *bi;
    int result;
    size_t datasize;

    /* Check arguments
       --------------- */
    MTX_VERIFY(ngen > 0);
    for (j = 0; j < ngen; ++j)
    {
        if (!MATRIX_IS_VALID(gen1[j]) || !MATRIX_IS_VALID(gen2[j]))
	    return -1;
	if (gen1[j]->nor != gen1[j]->noc)
	{
	    MTXERROR1("gen1[%d]: Matrix not square",j);
	    return -1;
	}
	if (gen2[j]->nor != gen2[j]->noc)
	{
	    MTXERROR1("gen2[%d]: Matrix not square",j);
	    return -1;
	}
	if (gen1[j]->fl != gen1[0]->fl || gen1[j]->nor != gen1[0]->nor)
	{
	    MTXERROR1("gen1[%d]: Incompatible matrix",j);
	    return -1;
	}
	if (gen2[j]->fl != gen1[0]->fl || gen2[j]->nor != gen1[0]->nor)
	{
	    MTXERROR1("gen2[%d]: Incompatible matrix",j);
	    return -1;
	}
    }

    /* Check if the dimensions are equal
       --------------------------------- */
    if (gen1[0]->nor != gen2[0]->nor)
	return 0;

    /* Make the idword on representation 2
       ----------------------------------- */
    wg = WgInit(ngen,gen2);
    word = WgMakeWord(wg,info1->idword);
    m = MatInsert(word,info1->idpol);
    MatFree(word);
    WgFree(wg);
    seed = MatNullSpace__(m);
    if (seed->nor != info1->spl)
    {	
	MatFree(seed);
	return 0;
    }

    /* Make the standard basis
       ----------------------- */
    b = SpinUp(seed,ngen,gen2,SF_FIRST|SF_CYCLIC|SF_STD);
    MatFree(seed);
    if (b->nor != b->noc)
    {
	MatFree(b);
	return 0;
    }
    bi = MatInv(b);

    /* Compare generators
       ------------------ */
    datasize = zsize(gen1[0]->nor);
    for (j = 0, result = 0; result == 0 && j < ngen; ++j)
    {
	matrix_t *g = MatDup(b);
	MatMul(g,gen2[j]);
	MatMul(g,bi);
	if (memcmp(g->d,gen1[j]->d,datasize))
	    result = 1;
	MatFree(g);
    }

    /* Clean up 
       -------- */
    if (trans != NULL && result == 0)
	*trans = b;
    else
	MatFree(b);
    MatFree(bi);

    return (result == 0);
}

==> MeatAxe-2.3.1.tar.gz: src/isisom.c <==
/* ============================= C MeatAxe ==================================
   File:        $Id: isisom.c,v 1.4 1998/09/17 17:03:53 mringe Exp $
   Comment:     Isomorphism test.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"
#include <string.h>


/*{{{ --------------------------------------------------------------------------
!function IsIsomorphic "Compare representations"
!section reps
!synopsis 
    int IsIsomorphic(int ngen, const matrix_t *gen1[], const CfInfo *info1, 
	const matrix_t *gen2[], matrix_t **trans, int use_pw)
!arg ngen
    Number of generators.
!arg gen1
    Generators for the first representation.
!arg info1
    Pointer to an |CfInfo| structure for the first representation.
!arg gen2
    Generators for the second representation.
!arg trans
    Buffer for basis transformation matrix, or |NULL|.
!arg use_pw
    If different from zero, use peak word instead of the identifying word 
    (see below).
!return
    1 if the representations are isomorphic, 0 if they are not isomorphic, and
    $-1$ on error.
!description
    This function decides if two irreducible representations are isomorphic.
    To compare the representations, the function needs an identifying word
    for the first representation, i.e., the fields |info1->idword|, 
    |info1->idpol| and |info1->spl| must be set, and the generators in 
    |gen1| must be in standard basis with respect to the identifying word.
    If |use_pw| is nonzero, the peak word is used instead of the idword.
    In this case, |gen1| must of course be in standard basis with respect 
    to the peak word.

    If the representations are isomorphic, and |trans| is not |NULL|, the
    basis transformation which makes the second representation identical
    to the first is stored into |*trans|. To be more precise, if $g_i$ is 
    the matrix in |gen1[i]|, $h_i$ the matrix in |gen2[i]|, and $T$ the 
    matrix returned in |trans|, then $Th_i T^{-1}=g_i$.
!seealso 
-------------------------------------------------------------------------- }}}*/

int IsIsomorphic(int ngen, const matrix_t *gen1[], const CfInfo *info1,
    const matrix_t *gen2[], matrix_t **trans, int use_pw)

{
    MTXCONTEXT(IsIsomorphic);
    int j;
    wgdata_t *wg;
    matrix_t *word, *m, *seed, *b, *bi;
    int result;
    size_t datasize;

    /* Check arguments
       --------------- */
    MTX_VERIFY(ngen > 0);
    for (j = 0; j < ngen; ++j)
    {
        if (!MATRIX_IS_VALID(gen1[j]) || !MATRIX_IS_VALID(gen2[j]))
	    return -1;
	if (gen1[j]->nor != gen1[j]->noc)
	{
	    MTXERROR1("gen1[%d]: Matrix not square",j);
	    return -1;
	}
	if (gen2[j]->nor != gen2[j]->noc)
	{
	    MTXERROR1("gen2[%d]: Matrix not square",j);
	    return -1;
	}
	if (gen1[j]->fl != gen1[0]->fl || gen1[j]->nor != gen1[0]->nor)
	{
	    MTXERROR1("gen1[%d]: Incompatible matrix",j);
	    return -1;
	}
	if (gen2[j]->fl != gen1[0]->fl || gen2[j]->nor != gen1[0]->nor)
	{
	    MTXERROR1("gen2[%d]: Incompatible matrix",j);
	    return -1;
	}
    }

    /* Check if the dimensions are equal
       --------------------------------- */
    if (gen1[0]->nor != gen2[0]->nor)
	return 0;

    /* Make the idword on representation 2
       ----------------------------------- */
    wg = WgInit(ngen,gen2);
    word = WgMakeWord(wg,use_pw ? info1->peakword : info1->idword);
    m = MatInsert(word,use_pw ? info1->peakpol : info1->idpol);
    MatFree(word);
    WgFree(wg);
    seed = MatNullSpace__(m);
    if (seed->nor != info1->spl)
    {	
	MatFree(seed);
	return 0;
    }

    /* Make the standard basis
       ----------------------- */
    b = SpinUp(seed,ngen,gen2,SF_FIRST|SF_CYCLIC|SF_STD);
    MatFree(seed);
    if (b->nor != b->noc)
    {
	MatFree(b);
	return 0;
    }
    bi = MatInv(b);

    /* Compare generators
       ------------------ */
    datasize = zsize(gen1[0]->nor);
    for (j = 0, result = 0; result == 0 && j < ngen; ++j)
    {
	matrix_t *g = MatDup(b);
	MatMul(g,gen2[j]);
	MatMul(g,bi);
	if (memcmp(g->d,gen1[j]->d,datasize))
	    result = 1;
	MatFree(g);
    }

    /* Clean up 
       -------- */
    if (trans != NULL && result == 0)
	*trans = b;
    else
	MatFree(b);
    MatFree(bi);

    return (result == 0);
}
